
#ifndef INC_GAMELOOP
#define INC_GAMELOOP
#endif

#ifndef INC_GAMELOOPIFC
#include "GameLoopIfc.h"
#endif
#ifndef INC_DISPLAYENGINE
#include "DisplayEngine.h"
#endif

#include <vector>

class SystemDialog;

//----------------------------------------------------------------------------

class GameLoop
{
public:
  GameLoop( DisplayEngine&, SystemDialog& );
  ~GameLoop() {}
  void config( const char*ini_fn );
  void add_view_port( ViewPortIfc& );
  void add_kbd_handler( KbdHndlIfc& );
  void add_gcl_handler( GameClockIfc& );
  void add_scroll_handler( ScrollingIfc& );
  void loop();
private:
  friend class GameLoopDebugView;
  int cur_skip;
  int max_skip;
  int rest_dur_ms;
  int normal_fps;
  DisplayEngine&display_engine;
  SystemDialog&sys_dialog;
  std::vector<GameClockIfc*> v_game_clock_hndls;
  std::vector<ViewPortIfc*> v_viewports;
  std::vector<KbdHndlIfc*> v_kbd_hndls;
  std::vector<ScrollingIfc*> v_scroll_hndls;
};

//----------------------------------------------------------------------------

class GameLoopDebugView : public ViewPortBase, public KbdHndlIfc
{
public:
  GameLoopDebugView( const ScreenRect&asr, const DisplayEngine&ade, const GameLoop&agl );
  virtual void display( BITMAP*const );
  virtual void handle_kbd( int scan_code, int shift_flags );
private:
  const DisplayEngine&display_engine;
  const GameLoop&game_loop;
  int what_to_show_;
};

//----------------------------------------------------------------------------

